<?php

  class report extends MY_Controller {

      protected $access_level;
      protected $total = array("all" => 0, "resolved" => 0);

      public function __construct() {
          parent::__construct();

          $this->vals["profile"] = $this->getUserInfo();

          if (!$this->userLogged()) {
              header("Location:./index.php?c=signIn");
              exit();
          }

          $this->access_level = $this->getAccessLevel();
      }

      protected function reportTicketPerDay($date_from, $date_to) {
          $ticket = array();
          $day_list = $this->dayList($date_from, $date_to);
          foreach ($day_list as $day) {
              $ticket[$day] = $this->total;
          }

          $this->load->model('ticket_model', 'ticket');
          $result = $this->ticket->allByDay($date_from, $date_to);
          foreach ($result as $val) {
              $ticket[$val['create_day']]['all'] = $val['counts'];
          }
          $result = $this->ticket->resolvedByDay($date_from, $date_to);
          foreach ($result as $val) {
              $ticket[$val['create_date']]['resolved'] = $val['counts'];
          }
          return $ticket;
      }

      protected function reportTicketPerMonth($month_from, $month_to) {
          $ticket = array();

          $month_list = $this->monthList($month_from, $month_to);
          foreach ($month_list as $month) {
              $ticket[$month] = $this->total;
          }

          $this->load->model('ticket_model', 'ticket');
          $result = $this->ticket->allByMonth($month_from, $month_to);
          foreach ($result as $val) {
              $val['create_month'] = sprintf("%02d", $val['create_month']);
              $ticket[$val['create_year'] . '-' . $val['create_month']]['all'] = $val['counts'];
          }

          $result = $this->ticket->resolvedByMonth($month_from, $month_to);
          foreach ($result as $val) {
              $val['create_month'] = sprintf("%02d", $val['create_month']);
              $ticket[$val['create_year'] . '-' . $val['create_month']]['resolved'] = $val['counts'];
          }
          return $ticket;
      }

      //----------------------------------------------------------

      public function index() {
          $this->vals["lang"] = $this->getLangArray("backend", $this->vals["profile"]["language"]);

          $select_type = (int) $this->input->get('select_type');
          $date_range = (int) $this->input->get('date_range');
          $report_type = (int) $this->input->get('report_type');

          if (empty($select_type)) {
              $select_type = 1;
          }

          if (empty($date_range)) {
              $date_range = 3;
          }

          if (empty($report_type)) {
              $report_type = 1;
          }

          $date_from = "";
          $date_to = "";

          if ($select_type == 1) {
              list($date_from, $date_to) = $this->getDateRange($date_range);
          } else {
              $date_from = $this->input->get('from') ? $this->input->get('from') : gmdate("Y-m-d", strtotime("-30 day"));
              $date_to = $this->input->get('to') ? $this->input->get('to') : gmdate("Y-m-d");
          }

          if ($date_from > $date_to) {
              list($date_from, $date_to) = array($date_to, $date_from);
          }

          // get ticket list
          $ticket = array();
          if ($report_type == 1) {
              $ticket = $this->reportTicketPerDay($date_from, $date_to);
          } else {
              $ticket = $this->reportTicketPerMonth($date_from, $date_to);
          }

          //-----------Chart Start --------------
          $this->load->library("FusionCharts");

          $arrCatNames = array();
          $arrData = array();
          $arrData[0][0] = "open";
          $arrData[0][1] = ""; // Dataset Parameters

          $arrData[1][0] = "resolved";
          $arrData[1][1] = ""; // Dataset Parameters

          if ($report_type == 1) {
              $i = 0;
              foreach ($ticket as $key => $count) {
                  $arrCatNames[$i] = gmdate("m-d", strtotime($key));
                  $arrData[0][2 + $i] = $count['all'] - $count['resolved'];
                  $arrData[1][2 + $i] = $count['resolved'];
                  $i++;
              }
          } else {
              $i = 0;
              foreach ($ticket as $key => $count) {
                  $arrCatNames[$i] = gmdate("Y-m", strtotime($key));
                  $arrData[0][2 + $i] = $count['all'] - $count['resolved'];
                  $arrData[1][2 + $i] = $count['resolved'];
                  $i++;
              }
          }

          $strParam = "caption=Ticket Report (day);formatNumberScale=1;rotateValues=0;rotateNames=1;decimalPrecision=0";

          $FC = new FusionCharts("StackedColumn3D", "1000", "300", "", "opaque");
          $FC->setSwfPath("./media/charts/");

          $FC->setChartParams($strParam);
          $FC->addChartDataFromArray($arrData, $arrCatNames);

          $this->vals["FC"] = $FC;

          //-----------Chart End --------------

          $this->vals['select_type'] = $select_type;
          $this->vals['date_range'] = $date_range;
          $this->vals['report_type'] = $report_type;
          $this->vals['date_from'] = $date_from;
          $this->vals['date_to'] = $date_to;

          $this->vals["channel"] = "report";
          $this->vals["sub_menu"] = "solved";
          $this->vals["menu"] = $this->load->view("backendMenu", $this->vals, true);

          $this->load->view('reportTicket', $this->vals);
      }

      protected function dayList($date_from, $date_to) {
          $date_from = strtotime($date_from);
          $date_to = strtotime($date_to);
          $day_lsit = array();
          $day_num = ceil(($date_to - $date_from) / 86400);
          for ($i = 0; $i <= $day_num; $i++) {
              $day_lsit[] = gmdate('Y-m-d', $date_from + $i * 86400);
          }
          return $day_lsit;
      }

      protected function dateWeek($offset_week, $business=false) {
          $year = gmdate('Y');
          $week = gmdate('W') + $offset_week;
          if ($week < 1) {
              $week = 52 + $week;
              $year--;
          } else if ($week > 52) {
              $week = $week - 52;
              $year++;
          }
          $week = sprintf("%02d", $week);
          $day_list = array();
          if ($business) {
              $day_list[0] = gmdate('Y-m-d', strtotime($year . "W" . $week . "1"));
              $day_list[1] = gmdate('Y-m-d', strtotime($year . "W" . $week . "5"));
          } else {
              $day_list[0] = gmdate('Y-m-d', strtotime($year . "W" . $week . "0"));
              $day_list[1] = gmdate('Y-m-d', strtotime($year . "W" . $week . "6"));
          }
          return $day_list;
      }

      protected function monthList($date_from, $date_to) {
          $date1 = explode("-", $date_from);
          $date2 = explode("-", $date_to);
          $month_list = array();
          for ($i = $date1[0]; $i < $date2[0]; $i++) {
              for ($j = 1; $j < 13; $j++) {
                  if ($i == $date1[0] && $j < $date1[1]) {
                      //
                  } else {
                      $month_list[] = $i . "-" . sprintf("%02d", $j);
                  }
              }
          }
          if ($date1[1] > $date2[1]) {
              list($date2[1], $date1[1]) = array($date1[1], $date2[1]);
          }
          for ($i = $date1[1]; $i <= $date2[1]; $i++) {
              $month_list[] = $date2[0] . "-" . sprintf("%02d", $i);
          }
          return $month_list;
      }

      protected function getDateRange($range) {
          $date_from = "";
          $date_to = "";

          switch ($range) {
              case 1:
                  // get today
                  $date_from = gmdate('Y-m-d');
                  $date_to = $date_from;
                  break;

              case 2:
                  // get yesterday
                  $date_from = gmdate('Y-m-d', strtotime("-1 day"));
                  $date_to = $date_from;
                  break;

              case 3:
                  // get this month
                  $month = gmdate('Y-m');
                  $date_from = $month . "-01";
                  $date_to = $month . "-" . gmdate("t");
                  break;

              case 4:
                  //get  last month
                  $gmdate = strtotime("-1 month");
                  $month = gmdate('Y-m', strtotime("-1 month"));
                  $date_from = $month . "-01";
                  $date_to = $month . "-" . gmdate("t", $gmdate);
                  break;

              case 5:
                  // get last 30 days
                  $date_from = gmdate('Y-m-d', strtotime("-30 day"));
                  $date_to = gmdate('Y-m-d');
                  break;

              case 6:
                  // get this week
                  list($date_from, $date_to) = $this->dateWeek(0);
                  $date_to = gmdate('Y-m-d');
                  break;

              case 7:
                  // get last week
                  list($date_from, $date_to) = $this->dateWeek(-1);
                  break;

              case 8:
                  // get this business week
                  list($date_from, $date_to) = $this->dateWeek(0, true);
                  $date_to = gmdate('Y-m-d');
                  break;

              case 9:
                  // get last business week
                  list($date_from, $date_to) = $this->dateWeek(-1, true);
                  break;

              case 10:
                  // get  this year
                  $date_from = gmdate('Y') . '-01-01';
                  $date_to = gmdate('Y-m-d');
                  break;

              case 11:
                  // get last year
                  $date_from = gmdate('Y') - 1 . '-01-01';
                  $date_to = gmdate('Y') - 1 . '-12-31';
                  break;

              case 12:
                  // get all time
                  $date_from = "2011-05-01";
                  $date_to = gmdate('Y-m-d');
                  break;

              default:
                  break;
          }

          return array($date_from, $date_to);
      }

  }

?>